@@ -121,12 +121,8 @@ Best Regards,'''))
 
     @api.model
     def _verify_fiscalyear_last_day(self, company_id, last_day, last_month):
-        company = self.browse(company_id)
-        last_day = last_day or (company and company.fiscalyear_last_day) or 31
-        last_month = last_month or (company and company.fiscalyear_last_month) or 12
-        current_year = datetime.now().year
-        last_day_of_month = calendar.monthrange(current_year, last_month)[1]
-        return last_day > last_day_of_month and last_day_of_month or last_day
+        # FIXME: Remove this method in master
+        return last_day
 
     @api.multi
     def compute_fiscalyear_dates(self, date):
@@ -138,7 +134,11 @@ Best Regards,'''))
         last_month = self.fiscalyear_last_month
         last_day = self.fiscalyear_last_day
         if (date.month < last_month or (date.month == last_month and date.day <= last_day)):
-            date = date.replace(month=last_month, day=last_day)
+            # FORWARD-PORT UP TO v11
+            if last_month == 2 and last_day == 29 and date.year % 4 != 0:
+                date = date.replace(month=last_month, day=28)
+            else:
+                date = date.replace(month=last_month, day=last_day)
         else:
             if last_month == 2 and last_day == 29 and (date.year + 1) % 4 != 0:
                 date = date.replace(month=last_month, day=28, year=date.year + 1)
