@@ -17,28 +17,42 @@ class ReportAgedPartnerBalance(models.AbstractModel):
         # Do an invoice and a payment and unreconcile. The amount will be nullified
         # By default, the partner wouldn't appear in this report.
         # The context key allow it to appear
+        # In case of a period_length of 30 days as of 2019-02-08, we want the following periods:
+        # Name       Stop         Start
+        # 1 - 30   : 2019-02-07 - 2019-01-09
+        # 31 - 60  : 2019-01-08 - 2018-12-10
+        # 61 - 90  : 2018-12-09 - 2018-11-10
+        # 91 - 120 : 2018-11-09 - 2018-10-11
+        # +120     : 2018-10-10
         periods = {}
         start = datetime.strptime(date_from, "%Y-%m-%d")
         for i in range(5)[::-1]:
             stop = start - relativedelta(days=period_length)
+            period_name = str((5-(i+1)) * period_length + 1) + '-' + str((5-i) * period_length)
+            period_stop = (start - relativedelta(days=1)).strftime('%Y-%m-%d')
+            if i == 0:
+                period_name = '+' + str(4 * period_length)
             periods[str(i)] = {
-                'name': (i!=0 and (str((5-(i+1)) * period_length) + '-' + str((5-i) * period_length)) or ('+'+str(4 * period_length))),
-                'stop': start.strftime('%Y-%m-%d'),
+                'name': period_name,
+                'stop': period_stop,
                 'start': (i!=0 and stop.strftime('%Y-%m-%d') or False),
             }
-            start = stop - relativedelta(days=1)
+            start = stop
 
         res = []
         total = []
         cr = self.env.cr
-        company_ids = self.env.context.get('company_ids', (self.env.user.company_id.id,))
+        user_company = self.env.user.company_id
+        user_currency = user_company.currency_id
+        ResCurrency = self.env['res.currency'].with_context(date=date_from)
+        company_ids = self._context.get('company_ids') or [user_company.id]
         move_state = ['draft', 'posted']
         if target_move == 'posted':
             move_state = ['posted']
         arg_list = (tuple(move_state), tuple(account_type))
         #build the reconciliation clause to see what partner needs to be printed
         reconciliation_clause = '(l.reconciled IS FALSE)'
-        cr.execute('SELECT debit_move_id, credit_move_id FROM account_partial_reconcile where create_date > %s', (date_from,))
+        cr.execute('SELECT debit_move_id, credit_move_id FROM account_partial_reconcile where max_date > %s', (date_from,))
         reconciled_after_date = []
         for row in cr.fetchall():
             reconciled_after_date += [row[0], row[1]]
@@ -68,7 +82,7 @@ class ReportAgedPartnerBalance(models.AbstractModel):
         partner_ids = [partner['partner_id'] for partner in partners if partner['partner_id']]
         lines = dict((partner['partner_id'] or False, []) for partner in partners)
         if not partner_ids:
-            return [], [], []
+            return [], [], {}
 
         # This dictionary will store the not due amount of all partners
         undue_amounts = {}
@@ -77,7 +91,7 @@ class ReportAgedPartnerBalance(models.AbstractModel):
                 WHERE (l.account_id = account_account.id) AND (l.move_id = am.id)
                     AND (am.state IN %s)
                     AND (account_account.internal_type IN %s)
-                    AND (COALESCE(l.date_maturity,l.date) > %s)\
+                    AND (COALESCE(l.date_maturity,l.date) >= %s)\
                     AND ((l.partner_id IN %s) OR (l.partner_id IS NULL))
                 AND (l.date <= %s)
                 AND l.company_id IN %s'''
@@ -88,15 +102,15 @@ class ReportAgedPartnerBalance(models.AbstractModel):
             partner_id = line.partner_id.id or False
             if partner_id not in undue_amounts:
                 undue_amounts[partner_id] = 0.0
-            line_amount = line.balance
-            if line.balance == 0:
+            line_amount = ResCurrency._compute(line.company_id.currency_id, user_currency, line.balance)
+            if user_currency.is_zero(line_amount):
                 continue
             for partial_line in line.matched_debit_ids:
                 if partial_line.max_date <= date_from:
-                    line_amount += partial_line.amount
+                    line_amount += ResCurrency._compute(partial_line.company_id.currency_id, user_currency, partial_line.amount)
             for partial_line in line.matched_credit_ids:
                 if partial_line.max_date <= date_from:
-                    line_amount -= partial_line.amount
+                    line_amount -= ResCurrency._compute(partial_line.company_id.currency_id, user_currency, partial_line.amount)
             if not self.env.user.company_id.currency_id.is_zero(line_amount):
                 undue_amounts[partner_id] += line_amount
                 lines[partner_id].append({
@@ -136,19 +150,19 @@ class ReportAgedPartnerBalance(models.AbstractModel):
             partners_amount = {}
             aml_ids = cr.fetchall()
             aml_ids = aml_ids and [x[0] for x in aml_ids] or []
-            for line in self.env['account.move.line'].browse(aml_ids):
+            for line in self.env['account.move.line'].browse(aml_ids).with_context(prefetch_fields=False):
                 partner_id = line.partner_id.id or False
                 if partner_id not in partners_amount:
                     partners_amount[partner_id] = 0.0
-                line_amount = line.balance
-                if line.balance == 0:
+                line_amount = ResCurrency._compute(line.company_id.currency_id, user_currency, line.balance)
+                if user_currency.is_zero(line_amount):
                     continue
                 for partial_line in line.matched_debit_ids:
                     if partial_line.max_date <= date_from:
-                        line_amount += partial_line.amount
+                        line_amount += ResCurrency._compute(partial_line.company_id.currency_id, user_currency, partial_line.amount)
                 for partial_line in line.matched_credit_ids:
                     if partial_line.max_date <= date_from:
-                        line_amount -= partial_line.amount
+                        line_amount -= ResCurrency._compute(partial_line.company_id.currency_id, user_currency, partial_line.amount)
 
                 if not self.env.user.company_id.currency_id.is_zero(line_amount):
                     partners_amount[partner_id] += line_amount
