@@ -56,7 +56,7 @@ class AccountPayment(models.Model):
     @api.onchange('amount','currency_id')
     def _onchange_amount(self):
         res = super(AccountPayment, self)._onchange_amount()
-        self.check_amount_in_words = self.currency_id.amount_to_text(self.amount)
+        self.check_amount_in_words = self.currency_id.amount_to_text(self.amount) if self.currency_id else ''
         return res
 
     def _check_communication(self, payment_method_id, communication):
@@ -119,3 +119,7 @@ class AccountPayment(models.Model):
         """ This method is a hook for l10n_xx_check_printing modules to implement actual check printing capabilities """
         raise UserError(_("There is no check layout configured.\nMake sure the proper check printing module is installed"
                           " and its configuration (in company settings > 'Configuration' tab) is correct."))
+
+    def set_check_amount_in_words(self):
+        for payment in self:
+            payment.check_amount_in_words = payment.currency_id.amount_to_text(payment.amount) if payment.currency_id else ''
