@@ -2,6 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import http, _
+from odoo.exceptions import UserError
 from odoo.addons.portal.controllers.portal import _build_url_w_params
 from odoo.http import request, route
 
@@ -27,7 +28,7 @@ class PaymentPortal(http.Controller):
         except:
             return False
 
-        if request.env.user == request.env.ref('base.public_user'):
+        if request.env.user._is_public():
             save_token = False # we avoid to create a token for the public user
 
         token = request.env['payment.token'].sudo()  # currently no support of payment tokens
@@ -94,3 +95,36 @@ class PaymentPortal(http.Controller):
                 return request.redirect(_build_url_w_params(error_url, params))
             params['success'] = 'pay_invoice'
         return request.redirect(_build_url_w_params(success_url, params))
+
+    @http.route('/invoice/pay/<int:invoice_id>/s2s_json_token_tx', type='json', auth='public')
+    def invoice_pay_token_json(self, invoice_id, pm_id=None, **kwargs):
+        """ Use a token to perform a s2s transaction """
+        invoice_sudo = request.env['account.invoice'].sudo().browse(invoice_id).exists()
+        if not invoice_sudo:
+            raise UserError(_('Faulty invoice'))
+
+        try:
+            token = request.env['payment.token'].sudo().browse(int(pm_id))
+        except (ValueError, TypeError):
+            token = False
+        token_owner = invoice_sudo.partner_id if request.env.user == request.env.ref('base.public_user') else request.env.user.partner_id
+        if not token or token.partner_id != token_owner:
+            raise UserError(_('Invalid token'))
+        # find an existing tx or create a new one
+        tx = request.env['payment.transaction'].sudo()._check_or_create_invoice_tx(
+            invoice_sudo,
+            token.acquirer_id,
+            payment_token=token,
+            tx_type='server2server',
+        )
+
+        # set the transaction id into the session
+        request.session['portal_invoice_%s_transaction_id' % invoice_sudo.id] = tx.id
+
+        # proceed to the payment
+        res = tx.with_context(off_session=False).confirm_invoice_token()
+        tx_info = tx._get_json_info()
+        return {
+            'tx_info': tx_info,
+            'redirect': '/my/invoices/%s' % (invoice_sudo.id),
+        }
\ No newline at end of file
