@@ -56,20 +56,18 @@ var BarcodeEvents = core.Class.extend(mixins.PropertiesMixin, {
         this.isChromeMobile = isMobile && navigator.userAgent.match(/Chrome/i);
 
         // Creates an input who will receive the barcode scanner value.
-        if (this.isChromeMobile) {
-            this.$barcodeInput = $('<input/>', {
-                name: 'barcode',
-                type: 'text',
-                css: {
-                    'position': 'fixed',
-                    'top': '50%',
-                    'transform': 'translateY(-50%)',
-                    'z-index': '-1',
-                },
-            });
-            // Avoid to show autocomplete for a non appearing input
-            this.$barcodeInput.attr('autocomplete', 'off');
-        }
+        this.$barcodeInput = $('<input/>', {
+            name: 'barcode',
+            type: 'text',
+            css: {
+                'position': 'fixed',
+                'top': '50%',
+                'transform': 'translateY(-50%)',
+                'z-index': '-1',
+            },
+        });
+        // Avoid to show autocomplete for a non appearing input
+        this.$barcodeInput.attr('autocomplete', 'off');
 
         this.__blurBarcodeInput = _.debounce(this._blurBarcodeInput, this.inputTimeOut);
     },
@@ -143,6 +141,8 @@ var BarcodeEvents = core.Class.extend(mixins.PropertiesMixin, {
             e.key === "ArrowUp" || e.key === "ArrowDown" ||
             e.key === "Escape" || e.key === "Tab" ||
             e.key === "Backspace" || e.key === "Delete" ||
+            e.key === "Home" || e.key === "End" ||
+            e.key === "PageUp" || e.key === "PageDown" ||
             e.key === "Unidentified" || /F\d\d?/.test(e.key)) {
             return true;
         } else {
@@ -218,7 +218,8 @@ var BarcodeEvents = core.Class.extend(mixins.PropertiesMixin, {
      * @param  {jQuery.Event} e keydown event
      */
     _listenBarcodeScanner: function (e) {
-        if (!$('input:text:focus, textarea:focus, [contenteditable]:focus').length) {
+        if ($(document.activeElement).not('input:text, textarea, [contenteditable], ' +
+            '[type="email"], [type="number"], [type="password"], [type="tel"], [type="search"]').length) {
             $('body').append(this.$barcodeInput);
             this.$barcodeInput.focus();
         }
@@ -261,11 +262,9 @@ var BarcodeEvents = core.Class.extend(mixins.PropertiesMixin, {
      * @private
      */
     _blurBarcodeInput: function () {
-        if (this.$barcodeInput) {
-            // Close the virtual keyboard on mobile browsers
-            // FIXME: actually we can't prevent keyboard from opening
-            this.$barcodeInput.val('').blur();
-        }
+        // Close the virtual keyboard on mobile browsers
+        // FIXME: actually we can't prevent keyboard from opening
+        this.$barcodeInput.val('').blur();
     },
 
     start: function(prevent_key_repeat){
