@@ -10,7 +10,7 @@ import uuid
 from datetime import datetime
 from subprocess import Popen, PIPE
 from collections import OrderedDict
-from odoo import fields, tools
+from odoo import fields, tools, SUPERUSER_ID
 from odoo.tools.pycompat import string_types, to_text
 from odoo.http import request
 from odoo.modules.module import get_resource_path
@@ -230,10 +230,11 @@ class AssetsBundle(object):
         self.env.cr.execute("""
              SELECT max(id)
                FROM ir_attachment
-              WHERE url like %s
+              WHERE create_uid = %s
+                AND url like %s
            GROUP BY datas_fname
            ORDER BY datas_fname
-         """, [url_pattern])
+         """, [SUPERUSER_ID, url_pattern])
         attachment_ids = [r[0] for r in self.env.cr.fetchall()]
         return self.env['ir.attachment'].sudo().browse(attachment_ids)
 
@@ -384,6 +385,7 @@ class AssetsBundle(object):
                 compiled = self.compile_css(cmd, source)
                 if not self.css_errors and old_attachments:
                     old_attachments.unlink()
+                    old_attachments = None
 
                 fragments = self.rx_css_split.split(compiled)
                 at_rules = fragments.pop(0)
