@@ -370,6 +370,10 @@ class FloatTimeConverter(models.AbstractModel):
     def value_to_html(self, value, options):
         sign = math.copysign(1.0, value)
         hours, minutes = divmod(abs(value) * 60, 60)
+        minutes = round(minutes)
+        if minutes == 60:
+            minutes = 0
+            hours += 1
         return '%02d:%02d' % (sign * hours, minutes)
 
 
