@@ -2,6 +2,7 @@ from . import test_acl
 from . import test_api
 from . import test_base
 from . import test_basecase
+from . import test_cache
 from . import test_db_cursor
 from . import test_expression
 from . import test_float
@@ -31,5 +32,7 @@ from . import test_user_has_group
 from . import test_view_validation
 from . import test_views
 from . import test_xmlrpc
+from . import test_res_partner
 from . import test_res_partner_bank
 from . import test_reports
+from . import test_non_regression
