@@ -5,7 +5,6 @@ import collections
 import json
 import os.path
 import re
-import sys
 
 from lxml import etree
 
@@ -481,12 +480,57 @@ class TestQWebNS(TransactionCase):
                 </t>
             """
         })
-        error_msg = "Can't convert 'int' object to str implicitly"
-        if sys.version_info >= (3, 6):
-            error_msg = "must be str, not int"
-        with self.assertRaisesRegexp(QWebException, error_msg):
+
+        try:
+            "" + 0
+        except TypeError as e:
+            error_msg = e.args[0]
+
+        with self.assertRaises(QWebException, msg=error_msg):
             view1.render()
 
+    def test_render_t_call_propagates_t_lang(self):
+        current_lang = 'en_US'
+        other_lang = 'fr_FR'
+
+        self.env['res.lang'].load_lang(lang=other_lang)
+
+        self.env['res.lang'].search([('code', '=', other_lang)], limit=1).write({
+            'active': True,
+            'decimal_point': '*',
+            'thousands_sep': '/'
+        })
+
+        view1 = self.env['ir.ui.view'].create({
+            'name': "callee",
+            'type': 'qweb',
+            'arch': u"""
+                <t t-name="base.callee">
+                    <t t-esc="9000000.00" t-options="{'widget': 'float', 'precision': 2}" />
+                </t>
+            """
+        })
+        self.env['ir.model.data'].create({
+            'name': 'callee',
+            'model': 'ir.ui.view',
+            'module': 'base',
+            'res_id': view1.id,
+        })
+
+        view2 = self.env['ir.ui.view'].create({
+            'name': "calling",
+            'type': 'qweb',
+            'arch': u"""
+                <t t-name="base.calling">
+                    <t t-call="base.callee" t-lang="'%s'" />
+                </t>
+            """ % other_lang
+        })
+
+        rendered = view2.with_context(lang=current_lang).render().strip()
+        self.assertEqual(rendered, b'9/000/000*00')
+
+
 from copy import deepcopy
 class FileSystemLoader(object):
     def __init__(self, path):
