@@ -9,6 +9,8 @@ var Domain = require('web.Domain');
 var FormController = require('web.FormController');
 var FormRenderer = require('web.FormRenderer');
 var FormView = require('web.FormView');
+var pyUtils = require('web.pyeval'); // do not forwardport this to 12.0
+var session  = require('web.session');
 var viewRegistry = require('web.view_registry');
 
 var _t = core._t;
@@ -243,7 +245,6 @@ FormRenderer.include({
      */
     _createController: function (params) {
         var self = this;
-        var context = params.context.eval();
         return this._rpc({
                 route: '/web/action/load',
                 params: {action_id: params.actionID}
@@ -253,18 +254,24 @@ FormRenderer.include({
                     // the action does not exist anymore
                     return $.when();
                 }
+                // tz and lang are saved in the custom view
+                // override the language to take the current one
+                var rawContext = new Context(params.context, action.context, {lang: session.user_context.lang});
+                var context = pyUtils.eval('context', rawContext);
+                var domain = params.domain || pyUtils.eval('domain', action.domain || '[]', action.context);
+                var viewType = params.viewType || action.views[0][1];
                 var view = _.find(action.views, function (descr) {
-                    return descr[1] === params.viewType;
-                });
-                return self.loadViews(action.res_model, params.context, [view])
+                    return descr[1] === viewType;
+                }) || [false, viewType];
+                return self.loadViews(action.res_model, rawContext, [view]) // use context instead of rawContext when forwardported to > 11.0
                            .then(function (viewsInfo) {
-                    var viewInfo = viewsInfo[params.viewType];
-                    var View = viewRegistry.get(params.viewType);
+                    var viewInfo = viewsInfo[viewType];
+                    var View = viewRegistry.get(viewType);
                     var view = new View(viewInfo, {
                         action: action,
                         context: context,
-                        domain: params.domain,
-                        groupBy: context.group_by,
+                        domain: domain,
+                        groupBy: context.group_by || [],
                         modelName: action.res_model,
                         hasSelectors: false,
                     });
