@@ -11,7 +11,7 @@ from odoo.addons.calendar.models.calendar import get_real_ids
 class Partner(models.Model):
     _inherit = 'res.partner'
 
-    calendar_last_notif_ack = fields.Datetime('Last notification marked as read from base Calendar')
+    calendar_last_notif_ack = fields.Datetime('Last notification marked as read from base Calendar', default=fields.Datetime.now)
 
     @api.multi
     def get_attendee_detail(self, meeting_id):
