@@ -358,7 +358,7 @@ class Escpos:
             else:
                 self._raw(string)
        
-        raw += S_RASTER_N
+        raw += S_RASTER_N.encode('utf-8')
         buffer = "%02X%02X%02X%02X" % (int((size[0]/size[1])/8), 0, size[1], 0)
         raw += codecs.decode(buffer, 'hex')
         buffer = ""
@@ -888,14 +888,23 @@ class Escpos:
             self._raw(PAPER_FULL_CUT)
 
 
-    def cashdraw(self, pin):
-        """ Send pulse to kick the cash drawer """
-        if pin == 2:
-            self._raw(CD_KICK_2)
-        elif pin == 5:
-            self._raw(CD_KICK_5)
-        else:
-            raise CashDrawerError()
+    def cashdraw(self, pin, tries=5):
+        """ Send pulse to kick the cash drawer
+
+        With some printers the drawer will not open after one pulse, for this reason we will check
+        the drawer status up to 'tries' times and send a new pulse if the drawer is still closed. If the
+        drawer status is open, we will stop sending pulses.
+        """
+        for i in range(tries):
+            if pin == 2:
+                self._raw(CD_KICK_2)
+            elif pin == 5:
+                self._raw(CD_KICK_5)
+            else:
+                raise CashDrawerError()
+
+            if i != tries - 1 and not self.get_printer_status()['printer']['drawer_pin_high']:
+                break
 
 
     def hw(self, hw):
