@@ -19,7 +19,7 @@ This module provides standard tools (account model, context manager and helpers)
     'qweb': [
         'static/src/xml/iap_templates.xml',
     ],
-    'auto_install': True,
+    'auto_install': False,
     # needed because dependencies can't be changed in a stable version
     # TODO in master: add web_settings_dashboard to depends and remove this
     'post_init_hook': '_install_web_settings_dashboard',
