@@ -80,7 +80,6 @@ class LivechatController(http.Controller):
     @http.route('/im_livechat/feedback', type='json', auth='public')
     def feedback(self, uuid, rate, reason=None, **kwargs):
         Channel = request.env['mail.channel']
-        Rating = request.env['rating.rating']
         channel = Channel.sudo().search([('uuid', '=', uuid)], limit=1)
         if channel:
             # limit the creation : only ONE rating per session
@@ -101,7 +100,11 @@ class LivechatController(http.Controller):
                 # if logged in user, set its partner on rating
                 values['partner_id'] = request.env.user.partner_id.id if request.session.uid else False
                 # create the rating
-                rating = Rating.sudo().create(values)
+
+                channel.write({
+                    'rating_ids': [(0, False, values)]
+                })
+                rating = channel.rating_ids[0]
             else:
                 rating = channel.rating_ids[0]
                 rating.write(values)
