@@ -13,9 +13,9 @@ Bu modül kurulduktan sonra, Muhasebe yapılandırma sihirbazı çalışır
     * Sihirbaz sizden hesap planı şablonu, planın kurulacağı şirket, banka hesap
       bilgileriniz, ilgili para birimi gibi bilgiler isteyecek.
     """,
-    'author': 'Ahmet Altınışık',
+    'author': 'Ahmet Altınışık, Can Tecim',
     'maintainer':'https://launchpad.net/~openerp-turkey',
-    'website':'https://launchpad.net/openerp-turkey',
+    'website':'https://launchpad.net/openerp-turkey, http://www.cantecim.com',
     'depends': [
         'account',
     ],
@@ -24,5 +24,6 @@ Bu modül kurulduktan sonra, Muhasebe yapılandırma sihirbazı çalışır
         'data/account_data.xml',
         'data/account_tax_template_data.xml',
         'data/account_chart_template_data.yml',
+        'data/res.country.state.csv'
     ],
 }
