@@ -389,7 +389,7 @@ var BasicComposer = Widget.extend(chat_mixin, {
         // Attachments
         this.AttachmentDataSet = new data.DataSetSearch(this, 'ir.attachment', this.context);
         this.fileupload_id = _.uniqueId('o_chat_fileupload');
-        this.set('attachment_ids', []);
+        this.set('attachment_ids', options.attachment_ids || []);
 
         // Mention
         this.mention_manager = new MentionManager(this);
@@ -449,6 +449,7 @@ var BasicComposer = Widget.extend(chat_mixin, {
         dom.autoresize(this.$input, {parent: this, min_height: this.options.input_min_height});
 
         // Attachments
+        this.render_attachments();
         $(window).on(this.fileupload_id, this.on_attachment_loaded);
         this.on("change:attachment_ids", this, this.render_attachments);
 
@@ -618,6 +619,7 @@ var BasicComposer = Widget.extend(chat_mixin, {
         });
         attachments = attachments.concat(upload_attachments);
         this.set('attachment_ids', attachments);
+        event.target.value = "";
     },
     on_attachment_loaded: function(event) {
         var self = this,
