@@ -29,6 +29,18 @@ odoo.define('payment.payment_form', function (require) {
             $('[data-toggle="tooltip"]').tooltip();
         },
 
+        disableButton: function (button) {
+            $(button).attr('disabled', true);
+            $(button).children('.fa-lock').removeClass('fa-lock');
+            $(button).prepend('<span class="o_loader"><i class="fa fa-refresh fa-spin"></i>&nbsp;</span>');
+        },
+
+        enableButton: function (button) {
+            $(button).attr('disabled', false);
+            $(button).children('.fa').addClass('fa-lock');
+            $(button).find('span.o_loader').remove();
+        },
+
         payEvent: function (ev) {
             ev.preventDefault();
             var form = this.el;
@@ -83,10 +95,7 @@ odoo.define('payment.payment_form', function (require) {
                         return;
                     }
 
-                    $(button).attr('disabled', true);
-                    $(button).children('.fa-plus-circle').removeClass('fa-plus-circle')
-                    $(button).prepend('<span class="o_loader"><i class="fa fa-refresh fa-spin"></i>&nbsp;</span>');
-
+                    this.disableButton(button);
                     var verify_validity = this.$el.find('input[name="verify_validity"]');
 
                     if (verify_validity.length>0) {
@@ -121,19 +130,15 @@ odoo.define('payment.payment_form', function (require) {
                             }
                         }
                         // here we remove the 'processing' icon from the 'add a new payment' button
-                        $(button).attr('disabled', false);
-                        $(button).children('.fa').addClass('fa-plus-circle')
-                        $(button).find('span.o_loader').remove();
+                        self.enableButton(button);
                     }).fail(function (message, data) {
                         // if the rpc fails, pretty obvious
-                        $(button).attr('disabled', false);
-                        $(button).children('.fa').addClass('fa-plus-circle')
-                        $(button).find('span.o_loader').remove();
+                        self.enableButton(button);
 
                         self.displayError(
                             _t('Server Error'),
                             _t("We are not able to add your payment method at the moment.") +
-                               data.data.message
+                               message.data.message
                         );
                     });
                 }
@@ -178,7 +183,7 @@ odoo.define('payment.payment_form', function (require) {
                             self.displayError(
                                 _t('Server Error'),
                                 _t("We are not able to redirect you to the payment form. ") +
-                                   data.data.message
+                                   message.data.message
                             );
                         });
                     }
@@ -191,6 +196,7 @@ odoo.define('payment.payment_form', function (require) {
                     }
                 }
                 else {  // if the user is using an old payment then we just submit the form
+                    this.disableButton(button);
                     form.submit();
                 }
             }
@@ -423,7 +429,7 @@ odoo.define('payment.payment_form', function (require) {
                 $acquirerForm = this.$('#o_payment_form_acq_' + acquirerID);
             }
 
-            if ($checkedRadio.length === 0) {
+            if (!$acquirerForm || $checkedRadio.length === 0) {
                 return new Dialog(null, {
                     title: _t('Error: ') + _.str.escapeHTML(title),
                     size: 'medium',
