@@ -18,6 +18,31 @@ from odoo.tools.pycompat import to_native
 
 _logger = logging.getLogger(__name__)
 
+# https://docs.adyen.com/developers/development-resources/currency-codes
+CURRENCY_CODE_MAPS = {
+    "BHD": 3,
+    "CVE": 0,
+    "DJF": 0,
+    "GNF": 0,
+    "IDR": 0,
+    "JOD": 3,
+    "JPY": 0,
+    "KMF": 0,
+    "KRW": 0,
+    "KWD": 3,
+    "LYD": 3,
+    "OMR": 3,
+    "PYG": 0,
+    "RWF": 0,
+    "TND": 3,
+    "UGX": 0,
+    "VND": 0,
+    "VUV": 0,
+    "XAF": 0,
+    "XOF": 0,
+    "XPF": 0,
+}
+
 
 class AcquirerAdyen(models.Model):
     _inherit = 'payment.acquirer'
@@ -27,6 +52,16 @@ class AcquirerAdyen(models.Model):
     adyen_skin_code = fields.Char('Skin Code', required_if_provider='adyen', groups='base.group_user')
     adyen_skin_hmac_key = fields.Char('Skin HMAC Key', required_if_provider='adyen', groups='base.group_user')
 
+    @api.model
+    def _adyen_convert_amount(self, amount, currency):
+        """
+        Adyen requires the amount to be multiplied by 10^k,
+        where k depends on the currency code.
+        """
+        k = CURRENCY_CODE_MAPS.get(currency.name, 2)
+        paymentAmount = int(tools.float_round(amount, k) * (10**k))
+        return paymentAmount
+
     @api.model
     def _get_adyen_urls(self, environment):
         """ Adyen URLs: yhpp: hosted payment page: pay.shtml for single, select.shtml for multiple """
@@ -120,12 +155,13 @@ class AcquirerAdyen(models.Model):
         import datetime
         from dateutil import relativedelta
 
+        paymentAmount = self._adyen_convert_amount(values['amount'], values['currency'])
         if self.provider == 'adyen' and len(self.adyen_skin_hmac_key) == 64:
             tmp_date = datetime.datetime.today() + relativedelta.relativedelta(days=1)
 
             values.update({
                 'merchantReference': values['reference'],
-                'paymentAmount': '%d' % int(tools.float_round(values['amount'], 2) * 100),
+                'paymentAmount': '%d' % paymentAmount,
                 'currencyCode': values['currency'] and values['currency'].name or '',
                 'shipBeforeDate': tmp_date.strftime('%Y-%m-%d'),
                 'skinCode': self.adyen_skin_code,
@@ -134,7 +170,7 @@ class AcquirerAdyen(models.Model):
                 'sessionValidity': tmp_date.isoformat('T')[:19] + "Z",
                 'resURL': urls.url_join(base_url, AdyenController._return_url),
                 'merchantReturnData': json.dumps({'return_url': '%s' % values.pop('return_url')}) if values.get('return_url', '') else False,
-                'shopperEmail': values.get('partner_email', ''),
+                'shopperEmail': values.get('partner_email') or values.get('billing_partner_email') or '',
             })
             values['merchantSig'] = self._adyen_generate_merchant_sig_sha256('in', values)
 
@@ -143,7 +179,7 @@ class AcquirerAdyen(models.Model):
 
             values.update({
                 'merchantReference': values['reference'],
-                'paymentAmount': '%d' % int(tools.float_round(values['amount'], 2) * 100),
+                'paymentAmount': '%d' % paymentAmount,
                 'currencyCode': values['currency'] and values['currency'].name or '',
                 'shipBeforeDate': tmp_date,
                 'skinCode': self.adyen_skin_code,
