@@ -192,11 +192,15 @@ var ProxyDevice  = core.Class.extend(mixins.PropertiesMixin,{
             // try harder when we remember a good proxy url
             found_url = this.try_hard_to_connect(localStorage.hw_proxy_url, options)
                 .then(null,function(){
-                    return self.find_proxy(options);
+                    if (window.location.protocol != 'https:'){
+                        return self.find_proxy(options);
+                    }
                 });
         }else{
             // just find something quick
-            found_url = this.find_proxy(options);
+            if (window.location.protocol != 'https:'){
+                found_url = this.find_proxy(options);
+            }
         }
 
         success = found_url.then(function(url){
@@ -248,16 +252,17 @@ var ProxyDevice  = core.Class.extend(mixins.PropertiesMixin,{
     // try several time to connect to a known proxy url
     try_hard_to_connect: function(url,options){
         options   = options || {};
-        var port  = ':' + (options.port || '8069');
+        var protocol = window.location.protocol;
+        var port = ( !options.port && protocol == "https:") ? ':443' : ':' + (options.port || '8069');
 
         this.set_connection_status('connecting');
 
         if(url.indexOf('//') < 0){
-            url = 'http://'+url;
+            url = protocol + '//' + url;
         }
 
         if(url.indexOf(':',5) < 0){
-            url = url+port;
+            url = url + port;
         }
 
         // try real hard to connect to url, with a 1sec timeout and up to 'retries' retries
@@ -273,11 +278,11 @@ var ProxyDevice  = core.Class.extend(mixins.PropertiesMixin,{
             .done(function(){
                 done.resolve(url);
             })
-            .fail(function(){
+            .fail(function(resp){
                 if(retries > 0){
                     try_real_hard_to_connect(url,retries-1,done);
                 }else{
-                    done.reject();
+                    done.reject(resp.statusText, url);
                 }
             });
             return done;
