@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from odoo.osv import expression
 from odoo import api, fields, models, _
 from odoo.addons import decimal_precision as dp
 from odoo.exceptions import UserError, ValidationError
@@ -108,6 +109,7 @@ class ProductAttributeLine(models.Model):
         # search on a m2o and one on a m2m, probably this will quickly become
         # difficult to compute - check if performance optimization is required
         if name and operator in ('=', 'ilike', '=ilike', 'like', '=like'):
-            args = expression.AND([['|', ('attribute_id', operator, name), ('value_ids', operator, name)], args])
-            return self.search(args, limit=limit).name_get()
+            args = args or []
+            domain = ['|', ('attribute_id', operator, name), ('value_ids', operator, name)]
+            return self.search(expression.AND([domain, args]), limit=limit).name_get()
         return super(ProductAttributeLine, self).name_search(name=name, args=args, operator=operator, limit=limit)
