@@ -104,7 +104,16 @@ class CustomerPortal(CustomerPortal):
         }
         # extends filterby criteria with project (criteria name is the project id)
         # Note: portal users can't view projects they don't follow
-        projects = request.env['project.project'].sudo().search([('privacy_visibility', '=', 'portal')])
+        partner = request.env.user.partner_id
+        domain_projects = [
+            '&',
+            ('privacy_visibility', '=', 'portal'),
+            '|',
+            ('message_partner_ids', 'child_of', [partner.commercial_partner_id.id]),
+            ('task_ids.message_partner_ids', 'child_of', [partner.commercial_partner_id.id])
+        ]
+
+        projects = request.env['project.project'].sudo().search(domain_projects)
         domain = [('project_id', 'in', projects.ids)]
         for proj in projects:
             searchbar_filters.update({
@@ -151,7 +160,7 @@ class CustomerPortal(CustomerPortal):
         # content according to pager and archive selected
         if groupby == 'project':
             order = "project_id, %s" % order  # force sort on project first to group by project in view
-        tasks = request.env['project.task'].search(domain, order=order, limit=self._items_per_page, offset=pager['offset'])
+        tasks = request.env['project.task'].search(domain, order=order, limit=self._items_per_page, offset=(page - 1) * self._items_per_page)
         request.session['my_tasks_history'] = tasks.ids[:100]
         if groupby == 'project':
             grouped_tasks = [request.env['project.task'].concat(*g) for k, g in groupbyelem(tasks, itemgetter('project_id'))]
