@@ -46,7 +46,12 @@ class StockMove(models.Model):
             if line.product_uom.id != line.product_id.uom_id.id:
                 price_unit *= line.product_uom.factor / line.product_id.uom_id.factor
             if order.currency_id != order.company_id.currency_id:
-                price_unit = order.currency_id.compute(price_unit, order.company_id.currency_id, round=False)
+                # DO NOT FORWARD-PORT! ONLY FOR V11!!!
+                # The date must be today, and not the date of the move since the move move is still
+                # in assigned state. However, the move date is the scheduled date until move is
+                # done, then date of actual move processing. See:
+                # https://github.com/odoo/odoo/blob/2f789b6863407e63f90b3a2d4cc3be09815f7002/addons/stock/models/stock_move.py#L36
+                price_unit = order.currency_id.with_context(date=fields.Date.context_today(self)).compute(price_unit, order.company_id.currency_id, round=False)
             return price_unit
         return super(StockMove, self)._get_price_unit()
 
@@ -60,6 +65,10 @@ class StockMove(models.Model):
         vals['purchase_line_id'] = self.purchase_line_id.id
         return vals
 
+    def _clean_merged(self):
+        super(StockMove, self)._clean_merged()
+        self.write({'created_purchase_line_id': False})
+
     def _action_cancel(self):
         for move in self:
             if move.created_purchase_line_id:
@@ -190,6 +199,6 @@ class PushedFlow(models.Model):
     _inherit = "stock.location.path"
 
     def _prepare_move_copy_values(self, move_to_copy, new_date):
-        res = super()._prepare_move_copy_values(move_to_copy, new_date)
+        res = super(PushedFlow, self)._prepare_move_copy_values(move_to_copy, new_date)
         res['purchase_line_id'] = None
         return res
