@@ -92,8 +92,7 @@ class MrpStockReport(models.TransientModel):
                     ('lot_id', '=', context.get('active_id')),
                     ('location_id.usage', '!=', 'internal'),
                     ('state', '=', 'done'),
-                    ('move_id.returned_move_ids', '=', False),
-                ])
+                ]).filtered(lambda s: not s.move_id.returned_move_ids)
                 res += self._lines(line_id, model_id=model_id, model='stock.move.line', level=level, parent_quant=parent_quant,
                                   stream=stream, obj_ids=move_ids)
                 quant_ids = self.env['stock.quant'].search([
@@ -108,8 +107,7 @@ class MrpStockReport(models.TransientModel):
                     ('lot_id', '=', context.get('active_id')),
                     ('location_dest_id.usage', '!=', 'internal'),
                     ('state', '=', 'done'),
-                    ('move_id.returned_move_ids', '=', False),
-                ])
+                ]).filtered(lambda s: not s.move_id.returned_move_ids)
                 res += self._lines(line_id, model_id=model_id, model='stock.move.line', level=level, parent_quant=parent_quant,
                                   stream=stream, obj_ids=move_ids)
                 quant_ids = self.env['stock.quant'].search([
@@ -139,10 +137,11 @@ class MrpStockReport(models.TransientModel):
         res_model = ''
         ref = ''
         res_id = False
-        if move_line.picking_id:
+        picking_id = move_line.picking_id or move_line.move_id.picking_id
+        if picking_id:
             res_model = 'stock.picking'
-            res_id = move_line.picking_id.id
-            ref = move_line.picking_id.name
+            res_id = picking_id.id
+            ref = picking_id.name
         elif move_line.move_id.inventory_id:
             res_model = 'stock.inventory'
             res_id = move_line.move_id.inventory_id.id
