@@ -22,7 +22,7 @@ internal transfer document is needed.
 """,
     'website': 'https://www.odoo.com/page/warehouse',
     'depends': ['purchase', 'sale_stock'],
-    'data': ['data/stock_data.xml'],
+    'data': ['data/stock_data.xml', 'views/sale_order_views.xml'],
     'test': [
         '../account/test/account_minimal_test.xml',
         '../stock_account/test/stock_valuation_account.xml',
