@@ -8,7 +8,7 @@ class StockPickingToBatch(models.TransientModel):
     _name = 'stock.picking.to.batch'
     _description = 'Add pickings to a batch picking'
 
-    batch_id = fields.Many2one('stock.picking.batch', string='Batch Picking', required=True, oldname="wave_id")
+    batch_id = fields.Many2one('stock.picking.batch', string='Batch Picking', oldname="wave_id")
 
     @api.multi
     def attach_pickings(self):
