@@ -1,4 +1,8 @@
 # -*- coding: utf-8 -*-
+try:
+    from unittest.mock import patch
+except ImportError:
+    from mock import patch
 
 from odoo.tests import common
 
@@ -104,12 +108,12 @@ class TestOnChange(common.TransactionCase):
 
     def test_onchange_one2many(self):
         """ test the effect of onchange() on one2many fields """
-        BODY = "What a beautiful day!"
         USER = self.env.user
 
         # create an independent message
-        message = self.Message.create({'body': BODY})
-        self.assertEqual(message.name, "[%s] %s" % ('', USER.name))
+        message1 = self.Message.create({'body': "ABC"})
+        message2 = self.Message.create({'body': "ABC"})
+        self.assertEqual(message1.name, "[%s] %s" % ('', USER.name))
 
         field_onchange = self.Discussion._onchange_spec()
         self.assertEqual(field_onchange.get('name'), '1')
@@ -126,12 +130,14 @@ class TestOnChange(common.TransactionCase):
             'moderator': False,
             'participants': [],
             'messages': [
-                (4, message.id),
+                (4, message1.id),
+                (4, message2.id),
+                (1, message2.id, {'body': "XYZ"}),
                 (0, 0, {
                     'name': "[%s] %s" % ('', USER.name),
-                    'body': BODY,
+                    'body': "ABC",
                     'author': USER.id,
-                    'size': len(BODY),
+                    'size': 3,
                     'important': False,
                 }),
             ],
@@ -141,14 +147,25 @@ class TestOnChange(common.TransactionCase):
         self.assertIn('messages', result['value'])
         self.assertEqual(result['value']['messages'], [
             (5,),
-            (1, message.id, {
+            (1, message1.id, {
                 'name': "[%s] %s" % ("Foo", USER.name),
+                'body': "ABC",
+                'author': USER.name_get()[0],
+                'size': 3,
+                'important': False,
+            }),
+            (1, message2.id, {
+                'name': "[%s] %s" % ("Foo", USER.name),
+                'body': "XYZ",          # this must be sent back
+                'author': USER.name_get()[0],
+                'size': 3,
+                'important': False,
             }),
             (0, 0, {
                 'name': "[%s] %s" % ("Foo", USER.name),
-                'body': BODY,
+                'body': "ABC",
                 'author': USER.name_get()[0],
-                'size': len(BODY),
+                'size': 3,
                 'important': False,
             }),
         ])
@@ -159,7 +176,8 @@ class TestOnChange(common.TransactionCase):
         result = self.Discussion.with_context(generate_dummy_message=True).onchange(values, 'name', one_level_fields)
         self.assertEqual(result['value']['messages'], [
             (5,),
-            (4, message.id),
+            (4, message1.id),
+            (4, message2.id),
             (0, 0, {}),
             (0, 0, {}),
         ])
@@ -251,26 +269,20 @@ class TestOnChange(common.TransactionCase):
             'name': partner2.name,
             'lines': [
                 (5,),
-                (1, line1.id, {'name': partner2.name,
-                               'partner': (partner2.id, partner2.name)}),
-                (0, 0, {'name': partner2.name,
-                        'partner': (partner2.id, partner2.name),
-                        'tags': [(5,)]}),
+                (1, line1.id, {
+                    'name': partner2.name,
+                    'partner': (partner2.id, partner2.name),
+                    'tags': [(5,)],
+                }),
+                (0, 0, {
+                    'name': partner2.name,
+                    'partner': (partner2.id, partner2.name),
+                    'tags': [(5,)],
+                }),
             ],
         })
 
         # do it again, but this time with a new tag on the second line
-        expected_new_tag_result = {
-            'name': partner2.name,
-            'lines': [
-                (5,),
-                (1, line1.id, {'name': partner2.name,
-                               'partner': (partner2.id, partner2.name)}),
-                (0, 0, {'name': partner2.name,
-                        'partner': (partner2.id, partner2.name),
-                        'tags': [(5,), (0, 0, {'name': 'Tag'})]}),
-            ],
-        }
         values = {
             'name': partner1.name,
             'partner': partner2.id,             # this one just changed
@@ -282,13 +294,35 @@ class TestOnChange(common.TransactionCase):
         self.env.cache.invalidate()
 
         result = multi.onchange(values, 'partner', field_onchange)
-        self.assertEqual(result['value'], expected_new_tag_result)
+        expected_value = {
+            'name': partner2.name,
+            'lines': [
+                (5,),
+                (1, line1.id, {
+                    'name': partner2.name,
+                    'partner': (partner2.id, partner2.name),
+                    'tags': [(5,)],
+                }),
+                (0, 0, {
+                    'name': partner2.name,
+                    'partner': (partner2.id, partner2.name),
+                    'tags': [(5,), (0, 0, {'name': 'Tag'})],
+                }),
+            ],
+        }
+        self.assertEqual(result['value'], expected_value)
 
         # ensure ID is not returned when asked and a many2many record is set to be created
         self.env.cache.invalidate()
 
         result = multi.onchange(values, 'partner', dict(field_onchange, **{'lines.tags.id': None}))
-        self.assertEqual(result['value'], expected_new_tag_result)
+        self.assertEqual(result['value'], expected_value)
+
+        # ensure inverse of one2many field is not returned
+        self.env.cache.invalidate()
+
+        result = multi.onchange(values, 'partner', dict(field_onchange, **{'lines.multi': None}))
+        self.assertEqual(result['value'], expected_value)
 
     def test_onchange_specific(self):
         """ test the effect of field-specific onchange method """
@@ -422,7 +456,14 @@ class TestOnChange(common.TransactionCase):
 
         self.assertEqual(
             result['value']['important_emails'],
-            [(5,), (1, email.id, {'name': u'[Foo Bar] %s' % USER.name})],
+            [(5,), (1, email.id, {
+                'name': u'[Foo Bar] %s' % USER.name,
+                'body': BODY,
+                'author': USER.name_get()[0],
+                'size': len(BODY),
+                'important': True,
+                'email_to': demo.email,
+            })],
         )
 
     def test_onchange_related(self):
@@ -453,3 +494,32 @@ class TestOnChange(common.TransactionCase):
         result = Message.onchange(value, ['message', 'message_name', 'message_currency'], field_onchange)
 
         self.assertEqual(result['value'], onchange_result)
+
+    def test_onchange_many2one_one2many(self):
+        """ Setting a many2one field should not read the inverse one2many. """
+        discussion = self.env.ref('test_new_api.discussion_0')
+        field_onchange = self.Message._onchange_spec()
+        self.assertEqual(field_onchange.get('discussion'), '1')
+
+        values = {
+            'discussion': discussion.id,
+            'name': "[%s] %s" % ('', self.env.user.name),
+            'body': False,
+            'author': self.env.uid,
+            'size': 0,
+        }
+
+        called = [False]
+        orig_read = type(discussion).read
+
+        def mock_read(self, fields=None, load='_classic_read'):
+            if discussion in self and 'messages' in (fields or ()):
+                called[0] = True
+            return orig_read(self, fields, load)
+
+        # changing 'discussion' on message should not read 'messages' on discussion
+        with patch.object(type(discussion), 'read', mock_read, create=True):
+            self.env.cache.invalidate()
+            self.Message.onchange(values, 'discussion', field_onchange)
+
+        self.assertFalse(called[0], "discussion.messages has been read")
