@@ -141,9 +141,11 @@ ListRenderer.include({
         if (self.currentRow !== null) {
             currentRowID = this.state.data[this.currentRow].id;
             currentWidget = this.allFieldWidgets[currentRowID][this.currentFieldIndex];
-            focusedElement = currentWidget.getFocusableElement().get(0);
-            if (currentWidget.formatType !== 'boolean') {
-                selectionRange = dom.getSelectionRange(focusedElement);
+            if (currentWidget) {
+                focusedElement = currentWidget.getFocusableElement().get(0);
+                if (currentWidget.formatType !== 'boolean') {
+                    selectionRange = dom.getSelectionRange(focusedElement);
+                }
             }
         }
 
@@ -186,9 +188,11 @@ ListRenderer.include({
                     // restore the cursor position
                     currentRowID = self.state.data[newRowIndex].id;
                     currentWidget = self.allFieldWidgets[currentRowID][self.currentFieldIndex];
-                    focusedElement = currentWidget.getFocusableElement().get(0);
-                    if (selectionRange) {
-                        dom.setSelectionRange(focusedElement, selectionRange);
+                    if (currentWidget) {
+                        focusedElement = currentWidget.getFocusableElement().get(0);
+                        if (selectionRange) {
+                            dom.setSelectionRange(focusedElement, selectionRange);
+                        }
                     }
                 });
             }
@@ -427,7 +431,7 @@ ListRenderer.include({
      */
     _moveToPreviousLine: function () {
         if (this.currentRow > 0) {
-            this._selectCell(this.currentRow - 1, this.columns.length - 1);
+            this._selectCell(this.currentRow - 1, this.columns.length - 1, {inc: -1});
         } else {
             this.unselectRow().then(this.trigger_up.bind(this, 'add_record'));
         }
@@ -467,7 +471,7 @@ ListRenderer.include({
     },
     /**
      * The renderer needs to support reordering lines.  This is only active in
-     * edit mode. The hasHandle attribute is used when there is a sequence
+     * edit mode. The handleField attribute is set when there is a sequence
      * widget.
      *
      * @override
@@ -475,7 +479,7 @@ ListRenderer.include({
      */
     _renderBody: function () {
         var $body = this._super();
-        if (this.hasHandle) {
+        if (this.handleField) {
             $body.sortable({
                 axis: 'y',
                 items: '> tr.o_data_row',
@@ -549,48 +553,48 @@ ListRenderer.include({
     _resequence: function (event, ui) {
         var self = this;
         var movedRecordID = ui.item.data('id');
-        var rows = this.state.data;
-        var row = _.findWhere(rows, {id: movedRecordID});
-        var index0 = rows.indexOf(row);
-        var index1 = ui.item.index();
-        var lower = Math.min(index0, index1);
-        var upper = Math.max(index0, index1) + 1;
+        self.unselectRow().then(function () {
+            var rows = self.state.data;
+            var row = _.findWhere(rows, {id: movedRecordID});
+            var index0 = rows.indexOf(row);
+            var index1 = ui.item.index();
+            var lower = Math.min(index0, index1);
+            var upper = Math.max(index0, index1) + 1;
 
-        var order = _.findWhere(self.state.orderedBy, {name: self.handleField});
-        var asc = !order || order.asc;
-        var reorderAll = false;
-        var sequence = (asc ? -1 : 1) * Infinity;
+            var order = _.findWhere(self.state.orderedBy, {name: self.handleField});
+            var asc = !order || order.asc;
+            var reorderAll = false;
+            var sequence = (asc ? -1 : 1) * Infinity;
 
-        // determine if we need to reorder all lines
-        _.each(rows, function (row, index) {
-            if ((index < lower || index >= upper) &&
-                ((asc && sequence >= row.data[self.handleField]) ||
-                 (!asc && sequence <= row.data[self.handleField]))) {
-                reorderAll = true;
-            }
-            sequence = row.data[self.handleField];
-        });
+            // determine if we need to reorder all lines
+            _.each(rows, function (row, index) {
+                if ((index < lower || index >= upper) &&
+                    ((asc && sequence >= row.data[self.handleField]) ||
+                     (!asc && sequence <= row.data[self.handleField]))) {
+                    reorderAll = true;
+                }
+                sequence = row.data[self.handleField];
+            });
 
-        if (reorderAll) {
-            rows = _.without(rows, row);
-            rows.splice(index1, 0, row);
-        } else {
-            rows = rows.slice(lower, upper);
-            rows = _.without(rows, row);
-            if (index0 > index1) {
-                rows.unshift(row);
+            if (reorderAll) {
+                rows = _.without(rows, row);
+                rows.splice(index1, 0, row);
             } else {
-                rows.push(row);
+                rows = rows.slice(lower, upper);
+                rows = _.without(rows, row);
+                if (index0 > index1) {
+                    rows.unshift(row);
+                } else {
+                    rows.push(row);
+                }
             }
-        }
 
-        var sequences = _.pluck(_.pluck(rows, 'data'), self.handleField);
-        var rowIDs = _.pluck(rows, 'id');
+            var sequences = _.pluck(_.pluck(rows, 'data'), self.handleField);
+            var rowIDs = _.pluck(rows, 'id');
 
-        if (!asc) {
-            rowIDs.reverse();
-        }
-        this.unselectRow().then(function () {
+            if (!asc) {
+                rowIDs.reverse();
+            }
             self.trigger_up('resequence', {
                 rowIDs: rowIDs,
                 offset: _.min(sequences),
@@ -616,6 +620,8 @@ ListRenderer.include({
      * @param {boolean} [options.force=false] if true, force selecting the cell
      *   even if seems to be already the selected one (useful after a re-
      *   rendering, to reset the focus on the correct field)
+     * @param {integer} [options.inc=1] the increment to use when searching for
+     *   the "next" possible cell (if the cell to select can't be selected)
      * @return {Deferred} fails if no cell could be selected
      */
     _selectCell: function (rowIndex, fieldIndex, options) {
@@ -639,7 +645,7 @@ ListRenderer.include({
             var oldFieldIndex = self.currentFieldIndex;
             self.currentFieldIndex = fieldIndex;
             fieldIndex = self._activateFieldWidget(record, fieldIndex, {
-                inc: 1,
+                inc: options.inc || 1,
                 wrap: wrap,
                 event: options && options.event,
             });
@@ -783,7 +789,7 @@ ListRenderer.include({
                 break;
             case 'previous':
                 if (this.currentFieldIndex > 0) {
-                    this._selectCell(this.currentRow, this.currentFieldIndex - 1, {wrap: false})
+                    this._selectCell(this.currentRow, this.currentFieldIndex - 1, {inc: -1, wrap: false})
                         .fail(this._moveToPreviousLine.bind(this));
                 } else {
                     this._moveToPreviousLine();
