@@ -881,7 +881,10 @@ eventHandler.attach = function (oLayoutInfo, options) {
 
             show_tooltip = true;
             setTimeout(function () {
-                if (!show_tooltip) return;
+                // Do not show tooltip on double-click and if there is already one
+                if (!show_tooltip || $target.attr('title') !== undefined) {
+                    return;
+                }
                 $target.tooltip({title: _t('Double-click to edit'), trigger: 'manuel', container: 'body'}).tooltip('show');
                 setTimeout(function () {
                     $target.tooltip('destroy');
