@@ -248,6 +248,13 @@ function applyOverDescendants(node, func) {
             func(node);
             applyOverDescendants(node, func);
         }
+        var $node = $(node);
+        if (node.nodeName === 'A' && $node.hasClass('btn') && !$node.children().length && $(node).parents('.o_outlook_hack').length)  {
+            node = $(node).parents('.o_outlook_hack')[0];
+        }
+        else if (node.nodeName === 'IMG' && $node.parent('p').hasClass('o_outlook_hack')) {
+            node = $node.parent()[0];
+        }
         node = node.nextSibling;
     }
 }
@@ -283,7 +290,7 @@ function classToStyle($editable) {
 
         // Outlook
         if (node.nodeName === 'A' && $target.hasClass('btn') && !$target.hasClass('btn-link') && !$target.children().length) {
-            var $hack = $('<table class="o_outlook_hack" style="display: inline-table;"><tr><td></td></tr></table>');
+            var $hack = $('<table class="o_outlook_hack" style="display: inline-table;vertical-align:middle"><tr><td></td></tr></table>');
             $hack.find('td')
                 .attr('height', $target.outerHeight())
                 .css({
@@ -304,6 +311,9 @@ function classToStyle($editable) {
                 $(node).remove();
             }
         }
+        else if (node.nodeName === 'IMG' && $target.hasClass('center-block')) {
+            $target.wrap('<p class="o_outlook_hack" style="text-align:center;margin:0"/>');
+        }
     });
 }
 
@@ -315,8 +325,8 @@ function classToStyle($editable) {
  */
 function styleToClass($editable) {
     // Outlook revert
-    $editable.find('table.o_outlook_hack').each(function () {
-        $(this).after($('a', this));
+    $editable.find('.o_outlook_hack').each(function () {
+        $(this).after($('a,img', this));
     }).remove();
 
     getMatchedCSSRules($editable[0]);
