@@ -113,7 +113,7 @@ var RunningTourActionHelper = core.Class.extend({
             $options.prop("selected", false).removeProp("selected");
             var $selectedOption = $options.filter(function () { return $(this).val() === text; });
             if ($selectedOption.length === 0) {
-                $selectedOption = $options.filter(function () { return $(this).text() === text; });
+                $selectedOption = $options.filter(function () { return $(this).text().trim() === text; });
             }
             $selectedOption.prop("selected", true);
             this._click(values);
