@@ -2,8 +2,10 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import werkzeug
+from functools import partial
 
 from odoo import exceptions, fields, http, _
+from odoo.tools import formatLang
 from odoo.http import request
 from odoo.addons.portal.controllers.portal import get_records_pager
 from odoo.addons.sale.controllers.portal import CustomerPortal
@@ -20,7 +22,7 @@ class CustomerPortal(CustomerPortal):
         except exceptions.AccessError:
             pass
         else:
-            if order_sudo.template_id and order_sudo.template_id.active:
+            if order_sudo.template_id and order_sudo.template_id.active and order_sudo.state in ['draft', 'sent']:
                 return request.redirect('/quote/%s/%s' % (order, access_token or ''))
         return super(CustomerPortal, self).portal_order_page(order=order, access_token=access_token, **kw)
 
@@ -127,8 +129,17 @@ class sale_quote(http.Controller):
             _message_post_helper(message=message, res_id=order_id, res_model='sale.order', **{'token': token} if token else {})
         return werkzeug.utils.redirect("/quote/%s/%s?message=2" % (order_id, token))
 
+    # Deprecated because override opportunities are **really** limited
+    # In fact it should be removed in master ASAP
     @http.route(['/quote/update_line'], type='json', auth="public", website=True)
     def update(self, line_id, remove=False, unlink=False, order_id=None, token=None, **post):
+        values = self.update_line_dict(line_id, remove, unlink, order_id, token, **post)
+        if values:
+            return [values['order_line_product_uom_qty'], values['order_amount_total']]
+        return values
+
+    @http.route(['/quote/update_line_dict'], type='json', auth="public", website=True)
+    def update_line_dict(self, line_id, remove=False, unlink=False, order_id=None, token=None, input_quantity=False, **kwargs):
         Order = request.env['sale.order'].sudo().browse(int(order_id))
         if token != Order.access_token:
             return request.render('website.404')
@@ -138,10 +149,27 @@ class sale_quote(http.Controller):
         if unlink:
             OrderLine.unlink()
             return False
-        number = -1 if remove else 1
-        quantity = OrderLine.product_uom_qty + number
+
+        if input_quantity is not False:
+            quantity = input_quantity
+        else:
+            number = -1 if remove else 1
+            quantity = OrderLine.product_uom_qty + number
+
+        if quantity < 0:
+            quantity = 0.0
         OrderLine.write({'product_uom_qty': quantity})
-        return [str(quantity), str(Order.amount_total)]
+        currency = Order.currency_id
+        format_price = partial(formatLang, request.env, digits=currency.decimal_places)
+
+        return {
+            'order_line_product_uom_qty': str(quantity),
+            'order_line_price_total': format_price(OrderLine.price_total),
+            'order_line_price_subtotal': format_price(OrderLine.price_subtotal),
+            'order_amount_total': format_price(Order.amount_total),
+            'order_amount_untaxed': format_price(Order.amount_untaxed),
+            'order_amount_tax': format_price(Order.amount_tax),
+        }
 
     @http.route(["/quote/template/<model('sale.quote.template'):quote>"], type='http', auth="user", website=True)
     def template_view(self, quote, **post):
@@ -232,3 +260,37 @@ class sale_quote(http.Controller):
         tx.confirm_sale_token()
         # redirect the user to the online quote
         return request.redirect('/quote/%s/%s' % (order_id, order.access_token))
+
+    @http.route('/quote/<int:order_id>/transaction/json_token', type='json', auth='public')
+    def payment_token_json(self, order_id, pm_id=None, **kwargs):
+        order = request.env['sale.order'].sudo().browse(order_id)
+        if not order or not order.order_line or pm_id is None:
+            raise exceptions.UserError(_('Nothing to pay/no token selected'))
+
+        # try to convert pm_id into an integer, if it doesn't work redirect the user to the quote
+        try:
+            pm_id = int(pm_id)
+        except ValueError:
+            raise exceptions.UserError(_('Faulty token value'))
+
+        # retrieve the token from its id
+        token = request.env['payment.token'].sudo().browse(pm_id)
+        if not token:
+            raise exceptions.UserError(_('Cannot find token'))
+
+        # find an already existing transaction
+        tx = request.env['payment.transaction'].sudo().search([('reference', '=', order.name)], limit=1)
+        # set the transaction type to server2server
+        tx_type = 'server2server'
+        # check if the transaction exists, if not then it create one
+        tx = tx._check_or_create_sale_tx(order, token.acquirer_id, payment_token=token, tx_type=tx_type)
+        # set the transaction id into the session
+        request.session['quote_%s_transaction_id' % order_id] = tx.id
+        # proceed to try the payment
+        tx.with_context(off_session=False).confirm_sale_token()
+        # send result in json
+        tx_info = tx._get_json_info()
+        return {
+            'tx_info': tx_info,
+            'redirect': '/quote/%s/%s' % (order_id, order.access_token),
+        }
