@@ -71,7 +71,10 @@ def _initialize_db(id, db_name, demo, lang, user_password, login='admin', countr
             if country_code:
                 countries = env['res.country'].search([('code', 'ilike', country_code)])
                 if countries:
-                    env['res.company'].browse(1).country_id = countries[0]
+                    comp_local = {'country_id': countries[0].id}
+                    if countries[0].currency_id:
+                        comp_local['currency_id'] = countries[0].currency_id.id
+                    env['res.company'].browse(1).write(comp_local)
 
             # update admin's password and lang and login
             values = {'password': user_password, 'lang': lang}
